#!/bin/sh -xv

#
# By Okajima, Jun. Tokyo, Japan. Apr. 2010.
# okajima@digitalinfra.co.jp
#
#
# ABSOLUTELY NO WARRANTY.
# GPL.
#
# 2010/5/10 Ver 0.01
#
#


ERR()
{
	local mes
	mes="ERR $(date) $*"
	echo $mes
	echo $mes >>$LOG
	exit 1
}


TIME()
{
	local cmd
	local ret
	cmd=$(which time)
	echo CMD $(date) $* >>$LOG
	$cmd -o $LOG -a $*
	ret=$?
	date >>$LOG
	echo "" >>$LOG
	echo "" >>$LOG
	return $ret
}


WGET()
{
	local fn
	fn=$(basename $1)
	[ -f $HOME/$fn ] && mv $HOME/$fn $HOME/$fn.$(date +"%Y%m%d%H%M%S")
	TIME wget -O $HOME/$fn $1 || ERR "wget"
}


FIND()
{
	sudo find / -anewer $OUT_DIR/newer_flag 2>/dev/null > $OUT_DIR/find.$1
	touch $OUT_DIR/newer_flag
	sleep 1.3
}


is_browser()
{
	echo $TARGET | grep -q browser
}


get_cpu_num()
{
	cat /proc/cpuinfo | grep ^processor | wc -l
}


get_jobs()
{
	local num
	local jobs
	num=$(get_cpu_num)
	jobs=$(($num+1))
	echo $jobs
}


browser_get_src_fn()
{
	local url
	url="http://build.chromium.org/buildbot/archives/chromium_tarball.html"
	wget -O - $url |
	grep window.location.href |
	grep -o "chromium.r[0-9]*.tgz"
}


browser_get_src_md5()
{
	local url
	url="http://build.chromium.org/buildbot/archives/${SRC_FN}.md5"
	WGET $url
	cat $SRC_FN.md5 | cut -d " " -f 1
}


browser_get_src_down()
{
	local url
	url="http://build.chromium.org/buildbot/archives/${SRC_FN}"
	WGET $url
}


check_md5()
{
	local md5
	md5=$(md5sum $SRC_FN | cut -d " " -f 1)
	[ $md5 = $SRC_MD5 ]
}


install_git()
{
	sudo apt-get update
	sudo apt-get --yes install git-core
	git --version
}


install_svn()
{
	sudo apt-get update
	sudo apt-get --yes install subversion
	svn --version
}


install_gclient()
{
	rm -f $HOME/depot_tools.tar.gz
	rm -rf $HOME/depot_tools
	WGET http://src.chromium.org/svn/trunk/tools/depot_tools.tar.gz
	tar -xzf depot_tools.tar.gz
	PATH=$PATH:$HOME/depot_tools
	gclient --version
}


check_multiverse()
{
	dpkg -l | grep msttcorefonts && return 0
	grep multiverse /etc/apt/sources.list && return 0
	cat <<EOT
Building Chromium Browser needs Microsoft TrueType fonts installed.
It is msttcorefonts/ttf-mscorefonts-installer in "multiverse".
Add "multiverse" to /etc/apt/sources.list
like :
deb http://us-east-1.ec2.archive.ubuntu.com/ubuntu/ lucid main universe multiverse
EOT
	ERR "multiverse"
}


browser_instal_build_64()
{
	uname -a | grep -q x86_64 || return 0
	sudo apt-get update
	sudo apt-get --yes install lib32bz2-dev
}


browser_install_build()
{
	local use_lib32
	#sudo apt-get update
	#sudo apt-get install ttf-mscorefonts-installer
	[ -f $HOME/install_build_okay ] && return 0
	check_multiverse
	WGET http://src.chromium.org/svn/trunk/src/build/install-build-deps.sh
	use_lib32="--lib32"
	uname -a | grep x86_64 && is_browser &&
	[ $TARGET = "browser-64" ] && use_lib32="--no-lib32"
	bash install-build-deps.sh \
	--no-syms --no-gold $use_lib32 ||
	ERR "install-build"
	dpkg -l >$OUT_DIR/dpkg-l.after.$DATE.txt || ERR dpkg-l
	touch $HOME/install_build_okay
}


check_target()
{
	echo $TARGET | grep -q x86 && return 0
	echo $TARGET | grep -q arm && return 0
	echo $TARGET | grep -q browser && return 0
	ERR "wrong target : $TARGET"
}


usage()
{
	cat <<EOT
	[OUT_PATH=] [SRC_PATH=] [JOBS=]  $PROGNAME TARGET
	OUT_PATH -> Default is \$HOME/\$TARGET
	SRC_PATH -> Default is \$HOME/\$TARGET
	JOBS -> Parallel compile. Default is num_cpu+1.
	TARGET ->
	x86-generic     Chromium OS for x86 32bit.
	amd-generic     Chromium OS for ARM.
	browser-32      Chromium Browser for x86 32bit.
	browser-64      Chromium Browser for AMD 64 ( needs 64bit linux)
	browser-cros    Chromium Browser for Chromium OS for x86 32bit.
	
	ex.
	OUT_PATH=\$HOME/out SRC_PATH=/mnt/ssd JOBS=3 $PROGNAME browser-64
	
	ABSOLUTELY NO WARRANTY. GPL.
EOT
	exit 1
}


set_gyp_defines()
{
	local flag_target_arch
	local flag_chromeos
	[ $TARGET = browser-32 ] && flag_target_arch="target_arch=ia32"
	[ $TARGET = browser-cros ] && flag_target_arch="target_arch=ia32"
	[ $TARGET = browser-cros ] && flag_chromeos="chromeos=1"
	#[ $TARGET = browser-64 ] && flag_target_arch="target_arch=x64"
	if [ $TARGET = browser-64 ]
	then
		flag_target_arch="x64"
		uname -a | grep -q x86_64 || ERR "needs 64bit linux"
	fi
	export GYP_DEFINES="$flag_target_arch $flag_chromeos"
}


get_ec2_meta()
{
	echo -n "$1 "
	wget -O - http://169.254.169.254/latest/meta-data/$1
}


get_ec2_data()
{
	uname -a | grep -q "-ec2" || return 0
	cat <<EOT
Amazon EC2 detected:
$(get_ec2_meta ami-id)
$(get_ec2_meta instance-type)
EOT
	return 0
}


init_all()
{
	[ $# -eq 1 ] || usage
	# Note that probably Koala might work, but no guarantee.
	grep "^DISTRIB_CODENAME=lucid$" /etc/lsb-release || ERR Use Lucid.
	#which apt-get || ERR "Only Ubuntu/Debian"
	TARGET=$1
	check_target
	is_browser && set_gyp_defines
	PATH=$PATH:$HOME/depot_tools
	DATE=$(date +"%Y%m%d%H%M%S")
	export GYP_GENERATORS=make
	#export GYP_DEFINES="target_arch=ia32"
	[ -n "$JOBS" ] || JOBS=$(get_jobs)
	SRC_DIR_NAME="CHROMIUM_${TARGET}_SRC"
	OUT_DIR_NAME="CHROMIUM_${TARGET}_${DATE}_OUT"
	[ -n "$SRC_PATH" ] && SRC_DIR="$SRC_PATH/$SRC_DIR_NAME"
	[ -n "$SRC_PATH" ] || SRC_DIR="$HOME/$SRC_DIR_NAME"
	[ -n "$SRC_PATH" ] || SRC_PATH="$HOME"
	[ -n "$OUT_PATH" ] && OUT_DIR="$OUT_PATH/$OUT_DIR_NAME"
	[ -n "$OUT_PATH" ] || OUT_DIR="$HOME/$OUT_DIR_NAME"
	[ -n "$OUT_PATH" ] || OUT_PATH="$HOME"
	SVN="home/chrome-svn/tarball/chromium"
	LOG=$OUT_DIR/CHROMIUM_${TARGET}_${DATE}.log
	BROWSER_BINARY="out/Release/chrome"
	SCRIPTS="$SRC_DIR/chromiumos.git/src/scripts"
	mkdir -p $OUT_DIR
	mkdir -p $SRC_DIR
	cp $PROGNAME $OUT_DIR/compile_browser.$DATE.sh
	dpkg -l >$OUT_DIR/dpkg-before.$DATE.txt || ERR dpkg-l
	cat <<EOT >> $LOG


-------start-----

cpuinfo:
$(cat /proc/cpuinfo)

meminfo:
$(cat /proc/meminfo)

uname -a:
$(uname -a)

date:
$(date)


SRC_DIR $SRC_DIR
OUT_DIR $OUT_DIR
GYP_DEFINES $GYP_DEFINES
GYP_GENERATORS $GYP_GENERATORS

mount:
$(mount)

cat /proc/mounts
$(cat /proc/mounts)

$(get_ec2_data)

EOT
	git --version || install_git || ERR "no git"
	svn --version || install_svn || ERR "no svn"
	gclient --version || install_gclient || ERR "no gclient"
}


do_browser_get_src()
{
	#local pfn
	#local files
	#local str
	#str=$SRC_DIR/home/chrome-svn/tarball/chromium/
	SRC_FN=$(browser_get_src_fn)
	[ -n "$SRC_FN" ] || ERR SRC_FN
	SRC_MD5=$(browser_get_src_md5)
	[ -n "$SRC_MD5" ] || ERR SRC_MD5
	check_md5 ||
	browser_get_src_down ||
	check_md5 || ERR "md5"
	[ -f $HOME/${SRC_MD5}_extracted ] ||
	TIME tar -C $SRC_DIR -xzf $HOME/$SRC_FN &&
	touch $HOME/${SRC_MD5}_extracted
	## mv home/chrome-svn/tarball/chromium -> $SRC_DIR/*
	#files="$files $(echo $str/*)"
	#files="$files $(echo $str/.[a-z]*)"
	#for pfn in $files
	#do
	#	mv $pfn $SRC_DIR
	#done
}


check_cros_deps()
{
	[ $TARGET = "browser-cros" ] || return 0
	grep -q cros_deps ${SRC_DIR}/${SVN}/.gclient && return 0
	cat <<EOT
You have to add Chromium OS dependancy in ${SRC_DIR}/$SVN/.gclient
like

before
---
     "src/chrome/tools/test/reference_build/chrome_mac": None,
     "src/chrome/tools/test/reference_build/chrome": None,
     "src/chrome/tools/test/reference_build/chrome_linux": None,
   }
 }
 <<--->> Insert here -->>>
]
---

after
---
     "src/chrome/tools/test/reference_build/chrome_mac": None,
     "src/chrome/tools/test/reference_build/chrome": None,
     "src/chrome/tools/test/reference_build/chrome_linux": None,
   }
 } ,
 { "name"        : "cros_deps",
   "url"         : "http://src.chromium.org/svn/trunk/cros_deps",
 } ,
]
EOT
	ERR "cros_deps"
}


do_browser_sync()
{
	cd $SRC_DIR/$SVN
	[ -f $SRC_DIR/$SVN/.gclient ] ||
	gclient config http://src.chromium.org/svn/trunk/src
	[ -f $SRC_DIR/$SVN/.gclient ] || ERR "no .gclient config."
	check_cros_deps
	cd $SRC_DIR/$SVN/src
	TIME gclient sync --force || ERR "sync"
	[ -f $SRC_DIR/$SVN/src/Makefile ] || TIME gclient runhooks --force || ERR "runhook"
	[ -f $SRC_DIR/$SVN/src/Makefile ] || ERR "no makefile"
	cd $HOME
}


do_browser_make()
{
	cd $SRC_DIR/$SVN/src
	TIME make BUILDTYPE=Release $BROWSER_BINARY -j$JOBS ||
	ERR "make"
	[ -f $SRC_DIR/$SVN/src/$BROWSER_BINARY ] || ERR "no binary"
	cd $HOME
}


browser_get_tar_list()
{
	cd $SRC_DIR/$SVN/src/out
	find -type f |
	grep -Fv /obj/ |
	grep -Fv obj.target |
	grep -Fv obj.host |
	grep -Fv deps/
	cd $HOME
}


do_browser_tar()
{
	local list
	list=$(browser_get_tar_list)
	tar -C $SRC_DIR/$SVN/src/out \
	-cf - $list | tar -C $OUT_DIR -xf -
}


do_cros_init()
{
	sudo apt-get update
	sudo apt-get --yes install binutils
}

do_cros_sync()
{
	rm -f ${SRC_DIR}/.gclient
	cd $SRC_DIR
	#gclient config git://src.chromium.org/git/chromiumos.git
	gclient config http://src.chromium.org/git/chromiumos.git
	TIME gclient sync || ERR "sync"
	cd $HOME
	FIND sync
}


do_cros_make_chroot()
{
	# a quick hack for grub issue. 2010/5/13 Okajima.
	TIME $SCRIPTS/make_chroot --jobs=$JOBS ||
	TIME $SCRIPTS/make_chroot --jobs=$JOBS ||
	ERR "make_chroot"
	FIND make_chroot
}


do_cros_cmd()
{
	# enter_chroot.sh needs "cd".
	cd $SCRIPTS
	TIME $SCRIPTS/enter_chroot.sh -- "$*" ||
	ERR "$1"
	cd $HOME
	FIND $1
}


do_cros_setup_board()
{
	#local str="chromiumos.git/chroot/usr/x86_64-pc-linux-gnu/gcc-bin/*/x86_64-pc-linux-gnu-gcc"
	local str="${SRC_DIR}/chromiumos.git/chroot/usr/local/bin/emerge-${TARGET}"
	#local cmd="/usr/local/bin/emerge-${TARGET}"
	# setup_board does not have --jobs option.
	#$SRC_DIR/$str --version ||
	#$str --version ||
	#do_cros_cmd $cmd --version ||
	[ -x $str ] ||
	do_cros_cmd ./setup_board "--board=$TARGET"
	[ -x $str ] || ERR "no emerge"
}


do_cros_passwd()
{
	do_cros_cmd ./enable_localaccount.sh okajima
	do_cros_cmd ./set_shared_user_password.sh
}


do_cros_build_emerges()
{
	local package
	for package in $EMERGES
	do
		do_cros_cmd emerge-$TARGET $package "--jobs=$JOBS"
	done
}

do_cros_build_packages()
{
	do_cros_cmd ./build_packages "--board=$TARGET --jobs=$JOBS"
}


do_cros_build_image()
{
	do_cros_cmd ./build_image "--board=$TARGET --jobs=$JOBS"
}


cros_get_image_dir()
{
	local dir
	local str
	dir=$SRC_DIR/chromiumos.git/src/build/images/$TARGET/
	str=$(ls -t $dir | head -1)
	echo $dir/$str/
}


cros_install_qemu()
{
	sudo apt-get update
	sudo apt-get --yes install qemu-kvm
}


do_cros_image_to_vmware()
{
	local dir
	dir=$(cros_get_image_dir)
	which qemu-img || cros_install_qemu || ERR "no qumu-img"
	#$SCRIPTS/image_to_vm.sh --board=$TARGET --from=$dir --to=$OUT_DIR
	(
	cd $SCRIPTS
	./image_to_vm.sh --board=$TARGET --from=$dir --to=$OUT_DIR
	)
}


do_cros_image_to_usb()
{
	local dir
	dir=$(cros_get_image_dir)
	$SCRIPTS/image_to_usb.sh --from=$dir --to=$OUT_DIR
}


do_tar()
{
	tar -C $OUT_PATH -czf ${OUT_DIR_NAME}.tgz $OUT_DIR_NAME
}


do_browser()
{
	#browser_install_build_64
	browser_install_build
	do_browser_get_src
	do_browser_sync
	do_browser_make
	do_browser_tar
	do_tar
}


do_cros_init_find()
{
	sudo find / 2>/dev/null > $OUT_DIR/find.start
	touch $OUT_DIR/newer_flag
	sleep 1.3
}


do_cros()
{
	do_cros_init
	do_cros_init_find
	do_cros_sync
	do_cros_make_chroot
	do_cros_setup_board
	do_cros_passwd
	do_cros_build_emerges
	do_cros_build_packages
	do_cros_build_image
	do_cros_image_to_vmware
	do_cros_image_to_usb
	do_tar
}


PROGNAME=$0
init_all $*
sleep 1
is_browser && do_browser
is_browser || do_cros



